function [ config ] = config( p_filename, p_force_load )
%LOAD_CONFIG Loads the given config file.
%
%   Parameters:
%   p_filename:       String, absolute or relative filepath to the target file.
%                     It is recommended to give the absolute filepath.
%
%   Returns:          Struct, containing the sections and member of the
%                     configuration file.
%

%% Initialize input parameter
config = false;

if ~exist('p_force_load', 'var')
    p_force_load = false;
end

%% Parameter check
parameter_error = false;

if (~ischar(p_filename))
    warning([mfilename ': Please check input parameter. p_filename is not a string!']);
    parameter_error = true;
elseif ~p_force_load
    % get filename details
    [pathstr, name, ext] = fileparts(p_filename);
    if (~strcmp(ext, '.conf'))
        warning([mfilename ': Please check input parameter. Wrong file extension! Should be .conf!']);
        parameter_error = true;
    end
end

if parameter_error
    return; % return if parameter check has not been passed successfully
else
    clear parameter_error;
end

%% Check if file exists
if exist(p_filename, 'file') ~= 2
    return;
end

%% Load configuration into variable

% initialize all required variables
config = {};
current_section = '';
comment_char = '%'; % character to indicate comments
section_char = '[';

% open file
file_id = fopen(p_filename);
fseek(file_id, 0, 'bof');


% read file line by line
current_line = fgetl(file_id);
while (ischar(current_line))
    if length(current_line) == 0 || strcmp(current_line(1), comment_char)
        current_line = fgetl(file_id);
        continue
    end
    
    switch current_line(1)
        case section_char % this indicates a new section
            current_line = strtrim(current_line);
            current_section = current_line(2:end-1);
            read_section
    end
    %current_line = fgetl(file_id);
end

fclose(file_id);

    function read_section
        % read complete section
        current_line = fgetl(file_id);
        while ischar(current_line) && ~isempty(current_line) ...
                && ~strcmp(current_line(1), section_char)
            if strcmp(current_line(1), comment_char) % it is a comment line
                current_line = fgetl(file_id);
                continue
            end
            
            % if it contains a comment char anywhere, just take the string
            % before it
            found_comment = strfind(current_line, comment_char);
            if ~isempty(found_comment)
                current_line = current_line(1:found_comment(1)-1);
            end
            
            % get variable name and value
            [token, remain] = strtok(current_line);
            % try to convert remain to a number
            remain_as_num = str2num(strtrim(remain));
            if (~isempty(remain_as_num))
                config.(current_section).(token(2:end)) = remain_as_num;
            else
                config.(current_section).(token(2:end)) = strtrim(remain);
            end
                    
            current_line = fgetl(file_id);
        end
    end


end

