function [ Boolean ] = export_attachment( Attachment, AbsoluteFilename )
%EXPORT_ATTACHMENT Exports the given attachment to the given filename.
%   Exports the attachment (type of com.sun.mail.util.BASE64DecoderStream)
%   to the given filename.
%
%   Parameters:
%   Attachment:         The attachment to be saved.
%   AbsoluteFilename:   The filename where the attachment is being
%                       exported to.
%
%   Returns:
%   Boolean:            True if successful.

%% Initialize return variable
Boolean = false;

%% Check parameter

if (~isa(Attachment, 'com.sun.mail.util.BASE64DecoderStream') ...
        || ~ischar(AbsoluteFilename))
    warning([mfilename ': Input parameter check failed!']);
    return
end

%% Create file and export attachment to it

[Path, ~, ~] = fileparts(AbsoluteFilename);

if (~isdir(Path)) % create it
    mkdir(Path);
end

% save every byte in attachment to file
File = java.io.File(AbsoluteFilename);
OutputStream = java.io.FileOutputStream(File);

Byte = Attachment.read();
while (Byte ~= -1)
    OutputStream.write(Byte);
    Byte = Attachment.read();
end

OutputStream.flush(); % prevent loosing bytes in write process
OutputStream.close();
Attachment.close();

Boolean = true;

end

